unit taskfile;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type

  TVariable = record
    Name: string;
    Value: string;
  end;

  TStep = record
    StepType: integer;
    Title: string;
    Delay: integer;
    TextToCopy: string;
    VariableName: string;
  end;

  TTaskFile = object
    FileName: string;
    Error: string;
    Steps: array of TStep;
    procedure Init;
    function Open: boolean;
    function Save: boolean;
    function moveUp(Index: integer): integer;
    function moveDown(Index: integer): integer;
    function addStep(Step: TStep): integer;
    function delStep(Index: integer): integer;
  end;

const
  StepTypes: array[0..2] of string = (
    'Копирование',
    'Отображение',
    'Ввод переменной'
    );

implementation

uses fpjson, jsonparser, jsonscanner;

procedure TTaskFile.Init;
begin
  FileName := '';
  SetLength(Steps, 0);
end;

function TTaskFile.Open: boolean;
var
  f: TextFile;
  s, s1: string;
  jo: TJSONParser;
  Data: TJSONData;
  Enum, StepEnum: TBaseJSONEnumerator;
  i: integer;
begin
  Error := '';
  s := '';
  Result := True;
  try
    AssignFile(f, FileName);
    reset(f);
    repeat
      readln(f, s1);
      s := s + s1;
    until EOF(f);
    CloseFile(f);
  except
    Error := 'Ошибка чтения файла.';
    Result := False;
    Exit;
  end;
  jo := TJSONParser.Create(s, DefaultOptions);
  try
    Data := jo.Parse;
  except
    FreeAndNil(jo);
    Error := 'Ошибка парсинга файла';
    Result := False;
    Exit;
  end;
  Init;
  // распарсили json - теперь раскладываем данные по свойствам
  try
    Enum := Data.GetEnumerator;
    while Enum.MoveNext do
    begin
      if Enum.Current.Key = 'steps' then
      begin
        if Enum.Current.Value.Count > 0 then
          for i := 0 to Enum.Current.Value.Count - 1 do
          begin
            SetLength(Steps, i + 1);
            Steps[i].Title := '';
            Steps[i].Delay := 0;
            Steps[i].StepType := 0;
            Steps[i].TextToCopy := '';
            Steps[i].VariableName := '';
            StepEnum := Enum.Current.Value.Items[i].GetEnumerator;
            while StepEnum.MoveNext do
            begin
              if StepEnum.Current.Key = 'title' then
                Steps[i].Title := StepEnum.Current.Value.AsString;
              if StepEnum.Current.Key = 'delay' then
                Steps[i].Delay := StepEnum.Current.Value.AsInteger;
              if StepEnum.Current.Key = 'type' then
                Steps[i].StepType := StepEnum.Current.Value.AsInteger;
              if StepEnum.Current.Key = 'text' then
                Steps[i].TextToCopy := StepEnum.Current.Value.AsString;
              if StepEnum.Current.Key = 'variableName' then
                Steps[i].VariableName := StepEnum.Current.Value.AsString;
            end;
            FreeAndNil(StepEnum);
          end;
      end;
    end;

  except
    FreeAndNil(Enum);
    FreeAndNil(jo);
    FreeAndNil(Data);
    Result := False;
    Error := 'Ошибка парсинга JSON';
  end;
  FreeAndNil(Enum);
  FreeAndNil(jo);
  FreeAndNil(Data);
end;

function TTaskFile.Save: boolean;
var
  SL: TStringList;
  i: integer;
  jo: TJSONObject;
  ja: TJSONArray;
begin
  Result := True;
  // составляем json
  jo := TJSONObject.Create([]);
  ja := TJSONArray.Create([]);
  if Length(Steps) > 0 then
    for i := 0 to Length(Steps) - 1 do
      ja.Add(TJSONObject.Create(['title', Steps[i].Title, 'delay',
        Steps[i].Delay, 'type', Steps[i].StepType, 'text', Steps[i].TextToCopy,
        'variableName', Steps[i].VariableName]));
  jo.Add('steps', ja);
  try
    SL := TStringList.Create;
    SL.Add(jo.AsJSON);
    SL.SaveToFile(FileName);
    SL.Free;
    FreeAndNil(jo);
  except
    on E: Exception do
    begin
      Error := E.Message;
      FreeAndNil(jo);
      Result := False;
    end;
  end;
end;

function TTaskFile.moveUp(Index: integer): integer;
var
  Step: TStep;
begin
  if ((Index < 1) or (Index > Length(Steps) - 1)) then
  begin
    Result := Index;
    Exit;
  end;
  Step := Steps[Index - 1];
  Steps[Index - 1] := Steps[Index];
  Steps[Index] := Step;
  Result := Index - 1;
end;

function TTaskFile.moveDown(Index: integer): integer;
var
  Step: TStep;
begin
  if ((Index < 0) or (Index >= Length(Steps) - 1)) then
  begin
    Result := Index;
    Exit;
  end;
  Step := Steps[Index + 1];
  Steps[Index + 1] := Steps[Index];
  Steps[Index] := Step;
  Result := Index + 1;
end;

function TTaskFile.addStep(Step: TStep): integer;
begin
  Result := Length(Steps);
  SetLength(Steps, Result + 1);
  Steps[Result] := Step;
end;

function TTaskFile.delStep(Index: integer): integer;
var
  i: integer;
begin
  if (Index < 0) or (Index > Length(Steps) - 1) then
  begin
    Result := -1;
    Exit;
  end;
  if (Index < Length(Steps) - 1) then
    for i := Index + 1 to Length(Steps) - 1 do
      Steps[i - 1] := Steps[i];
  SetLength(Steps, Length(Steps) - 1);
  if (Index > Length(Steps) - 1) then
    Result := Length(Steps) - 1
  else
    Result := Index;
end;

end.
